@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2017 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2006-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -1035,10 +1035,11 @@ relabel_menus (NSObject *v, NSString *old_str, NSString *new_str)
     if (name) [result addObject: name];
   }
 
-  if (! [result count])
-    result = 0;
-
-  return result;
+  if (result && [result count])
+    return [result sortedArrayUsingSelector:
+                     @selector(localizedCaseInsensitiveCompare:)];
+  else
+    return 0;
 }
 
 
@@ -1416,7 +1417,7 @@ FAIL:
                                   backing:NSBackingStoreBuffered
                                     defer:YES
                                    screen:screen];
-  [win setMinSize:[win frameRectForContentRect:rect].size];
+//  [win setMinSize:[win frameRectForContentRect:rect].size];
   [[win contentView] addSubview: (pbox ? (NSView *) pbox : (NSView *) sv)];
   [pbox release];
 
