@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1998-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1998-2017 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -168,7 +168,7 @@ main (int argc, char **argv)
   progname = argv[0];
   {
     char *s = strrchr(progname, '/');
-    if (*s) strcpy (progname, s+1);
+    if (*s) progname = s+1;
   }
 
   if (argc != 2) goto USAGE;
