@@ -143,8 +143,8 @@ static void render_light_map(struct state *st, fireshell *fs)
 	{
 		for (x = 0; x < st->width; x += 2, v += SHELLCOUNT)
 		{
-			float f;
-			f = sqrtf((fs->cx - x) * (fs->cx - x) + (fs->cy - y) * (fs->cy - y)) + 4.0;
+			double f;
+			f = sqrt((fs->cx - x) * (fs->cx - x) + (fs->cy - y) * (fs->cy - y)) + 4.0;
 			f = FLASH_ZOOM / f;
 			f += pow(f,0.1) * frand(0.0001); /* dither */
 			st->light_map[v] = f;
@@ -571,11 +571,13 @@ static void resize(struct state *st)
 	XSync(st->dpy, 0);
 	if (st->xim)
 	{
-		if (st->xim->data == (char *)st->palaka2) st->xim->data = NULL;
+		/* if (st->xim->data == (char *)st->palaka2) */
+                st->xim->data = NULL;
 		XDestroyImage(st->xim);
 		XSync(st->dpy, 0);
 		free(st->mem2);
 		free(st->mem1);
+                st->xim = 0;
 	}
 	st->xim = XCreateImage(st->dpy, xwa.visual, xwa.depth, ZPixmap, 0, 0,
 	                       st->width, st->height, 8, 0);
@@ -841,6 +843,14 @@ fireworkx_free (Display *dpy, Window window, void *closure)
 	free(st->mem1);
 	free(st->fireshell_array->fpix);
 	free(st->fireshell_array);
+        if (st->xim) {
+          st->xim->data = NULL;
+          XDestroyImage (st->xim);
+        }
+	if (st->light_map) free(st->light_map);
+        if (st->colors) free (st->colors);
+        XFreeGC (dpy, st->gc);
+        free(st);
 }
 
 static const char *fireworkx_defaults [] =
