@@ -56,8 +56,8 @@ static const char sccsid[] = "@(#)flag.c	4.02 97/04/01 xlockmore";
 # define flag_handle_event 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 
-#include "xpm-pixmap.h"
-#include "images/bob.xbm"
+#include "ximage-loader.h"
+#include "images/gen/bob_png.h"
 
 #else  /* !STANDALONE */
 # include "xlock.h"					/* from the xlockmore distribution */
@@ -196,8 +196,9 @@ make_flag_bits(ModeInfo *mi)
   char *bitmap_name = get_string_resource (dpy, "bitmap", "Bitmap");
   char *text = get_string_resource (dpy, "text", "Text");
 
-#ifdef HAVE_JWXYZ
-  bitmap_name = 0;  /* #### always use default */
+#ifdef HAVE_JWXYZ  /* always use default */
+  if (bitmap_name) free (bitmap_name);
+  bitmap_name = 0;
 #endif
 
   /* If neither a bitmap nor text are specified, randomly select either
@@ -224,10 +225,11 @@ make_flag_bits(ModeInfo *mi)
       int width = 0;
       int height = 0;
 
-      bitmap = xpm_file_to_pixmap (dpy, MI_WINDOW (mi), bitmap_name,
-                                   &width, &height, 0);
+      bitmap = file_to_pixmap (dpy, MI_WINDOW (mi), bitmap_name,
+                               &width, &height, 0);
 	  if (bitmap)
 		{
+          if (fp->image) XDestroyImage (fp->image);
 		  fp->image = XGetImage(dpy, bitmap, 0, 0, width, height, ~0L,
 								ZPixmap);
 		  XFreePixmap(dpy, bitmap);
@@ -256,6 +258,7 @@ make_flag_bits(ModeInfo *mi)
 		  struct utsname uts;
 		  if (uname (&uts) < 0)
 			{
+              if (text) free (text);
 			  text = strdup("uname() failed");
 			}
 		  else
@@ -263,6 +266,7 @@ make_flag_bits(ModeInfo *mi)
 			  char *s;
 			  if ((s = strchr(uts.nodename, '.')))
 				*s = 0;
+              if (text) free (text);
 			  text = (char *) malloc(strlen(uts.nodename) +
 									 strlen(uts.sysname) +
 									 strlen(uts.version) +
@@ -280,11 +284,14 @@ make_flag_bits(ModeInfo *mi)
                   char *s, buf[255];
 
                   while (fgets (buf, sizeof(buf)-1, f)) {
-#                   define GRAB(S,V)					\
-                    if (strstr(buf, S)) {					\
-                      fgets (buf, sizeof(buf)-1, f);			\
-                      if ((s = strchr (buf, '>'))) V = strdup(s+1); 	\
-                      if ((s = strchr (V, '<'))) *s = 0;		 	\
+#                   define GRAB(S,V)                                    \
+                    if (strstr(buf, S)) {                               \
+                      fgets (buf, sizeof(buf)-1, f);                    \
+                      if ((s = strchr (buf, '>'))) {                    \
+                        if (V) free (V);                                \
+                        V = strdup(s+1);                                \
+                      }                                                 \
+                      if ((s = strchr (V, '<'))) *s = 0;                \
                     }
                     GRAB ("ProductName", pn)
                     GRAB ("ProductBuildVersion", pbv)
@@ -298,6 +305,9 @@ make_flag_bits(ModeInfo *mi)
                 else
                   sprintf(text, "%s\n%s %s",
                           uts.nodename, uts.sysname, uts.release);
+                if (pbv) free (pbv);
+                if (pn) free (pn);
+                if (puvv) free (puvv);
               }
 # else
 			  sprintf(text, "%s\n%s %s",
@@ -305,6 +315,7 @@ make_flag_bits(ModeInfo *mi)
 # endif /* special system types */
 			}
 #else	/* !HAVE_UNAME */
+          if (text) free (text);
 # ifdef VMS
 		  text = strdup(getenv("SYS$NODE"));
 # else
@@ -320,14 +331,9 @@ make_flag_bits(ModeInfo *mi)
 
 	  text2 = strdup(text);
 
-	  if (!fn) fn = def_fn;
-      font = XLoadQueryFont (dpy, fn);
-      if (! font)
-		{
-		  fprintf(stderr, "%s: unable to load font %s; using %s\n",
-				  progname, fn, def_fn);
-		  font = XLoadQueryFont (dpy, def_fn);
-		}
+	  if (!fn) fn = strdup (def_fn);
+      font = load_font_retry (dpy, fn);
+      free (fn);
 
 	  memset(&overall, 0, sizeof(overall));
 	  token = text;
@@ -377,10 +383,10 @@ make_flag_bits(ModeInfo *mi)
 		  lines++;
 		}
 	  free(text2);
-	  XUnloadFont(dpy, font->fid);
-	  XFree((XPointer) font);
+	  XFreeFont(dpy, font);
 	  XFreeGC(dpy, gc);
 
+      if (fp->image) XDestroyImage (fp->image);
 	  fp->image = XGetImage(dpy, bitmap, 0, 0, width, height, 1L, XYPixmap);
 	  XFreePixmap(dpy, bitmap);
 	}
@@ -388,13 +394,26 @@ make_flag_bits(ModeInfo *mi)
 
   if (! fp->image)
 	{
-      char *bits = (char *) malloc (sizeof(bob_bits));
-      memcpy (bits, bob_bits, sizeof(bob_bits));
+      XImage *im = image_data_to_ximage (dpy, MI_VISUAL(mi),
+                                       bob_png, sizeof(bob_png));
+      int x, y;
+
 	  fp->image = XCreateImage (dpy, MI_VISUAL(mi), 1, XYBitmap, 0,
-								bits, bob_width, bob_height,
-								8, 0);
-	  fp->image->byte_order = LSBFirst;
-	  fp->image->bitmap_bit_order = LSBFirst;
+								0, im->width, im->height, 8, 0);
+      fp->image->data = malloc (fp->image->bytes_per_line * fp->image->height);
+
+      /* Convert deep image to 1 bit */
+      for (y = 0; y < im->height; y++)
+        {
+          for (x = 0; x < im->width; x++)
+            {
+              unsigned long p = XGetPixel (im, x, im->height-y-1);
+              if (! (p & 0xFF000000)) p = ~0;  /* alpha -> white */
+              p = (p >> 16) & 0xFF;            /* red */
+              XPutPixel (fp->image, x, y, p > 0x7F ? 0 : 1);
+            }
+        }
+      XDestroyImage (im);
 	}
 
   if (bitmap_name)
@@ -413,6 +432,7 @@ make_flag_bits(ModeInfo *mi)
   int w = flag_width;
   int h = flag_height;
   int i = 0;
+  if (fp->image) XDestroyImage (fp->image);
   fp->image =
 	XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi),
 				 1, XYBitmap, 0,					/* dpth, fmt, offset */
@@ -536,15 +556,10 @@ draw_flag(ModeInfo * mi)
 ENTRYPOINT void
 free_flag(ModeInfo * mi)
 {
-	int         screen = MI_SCREEN(mi);
-
-	if (flags == NULL)
-		return;
-
-	if (flags[screen].cache && flags[screen].dbufp)
-		XFreePixmap(MI_DISPLAY(mi), flags[screen].cache);
-	if (flags[screen].image)
-	  XDestroyImage(flags[screen].image);
+	flagstruct *fp = &flags[MI_SCREEN(mi)];
+	if (fp->cache && fp->dbufp)
+		XFreePixmap(MI_DISPLAY(mi), fp->cache);
+	if (fp->image) XDestroyImage(fp->image);
 }
 
 #ifndef STANDALONE
