@@ -459,6 +459,8 @@ static void
 flame_free (Display *dpy, Window window, void *closure)
 {
   struct state *st = (struct state *) closure;
+  XFreeGC (dpy, st->gc);
+  free (st->colors);
   free (st);
 }
 
