@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2003-2016 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2003-2017 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -164,6 +164,36 @@ pick_font_size (state *s)
 }
 
 
+#ifdef HAVE_JWXYZ
+
+static char *
+append_font_name(Display *dpy, char *dest, const XFontStruct *font)
+{
+  int i;
+  for (i = 0; i != font->n_properties; ++i) {
+    if (font->properties[i].name == XA_FONT) {
+      char *suffix = XGetAtomName (dpy, font->properties[i].card32);
+      int L = strlen(suffix);
+      strcpy(dest, suffix);
+      free (suffix);
+      return dest + L;
+    }
+  }
+
+  dest[0] = '?';
+  dest[1] = 0;
+  return dest + 1;
+
+/*
+  float s;
+  const char *n = jwxyz_nativeFontName (font->fid, &s);
+  return dest + sprintf (dest, "%s %.1f", n, s);
+ */
+}
+
+#endif
+
+
 /* Finds the set of scalable fonts on the system; picks one;
    and loads that font in a random pixel size.
    Returns False if something went wrong.
@@ -367,9 +397,12 @@ pick_font_1 (state *s, sentence *se)
   strcpy (pattern2, pattern);
 # ifdef HAVE_JWXYZ
   {
-    float s;
-    const char *n = jwxyz_nativeFontName (se->xftfont->xfont->fid, &s);
-    sprintf (pattern2 + strlen(pattern2), " (%s %.1f)", n, s);
+    char *out = pattern2 + strlen(pattern2);
+    out[0] = ' ';
+    out[1] = '(';
+    out = append_font_name (s->dpy, out + 2, se->xftfont->xfont);
+    out[0] = ')';
+    out[1] = 0;
   }
 # endif
 
@@ -737,6 +770,7 @@ free_word (state *s, word *w)
   if (w->text)   free (w->text);
   if (w->pixmap) XFreePixmap (s->dpy, w->pixmap);
   if (w->mask)   XFreePixmap (s->dpy, w->mask);
+  free (w);
 }
 
 
@@ -829,6 +863,7 @@ split_words (state *s, sentence *se)
           char *t2 = chars[k];
           word *w2 = new_word (s, se, t2, True);
           words2[j++] = w2;
+          free (t2);
 
           w2->x = x;
           w2->y = y;
@@ -1188,7 +1223,7 @@ populate_sentence (state *s, sentence *se)
               y + se->xftfont->ascent + se->xftfont->descent > s->xgwa.height)
             {
               unread_word (s, w);
-              free (w);
+              w = 0;
               /* done = True; */
               break;
             }
@@ -1622,11 +1657,7 @@ fontglide_draw_metrics (state *s)
 
   strcpy (fn2, fn);
 # ifdef HAVE_JWXYZ
-  {
-    float ss;
-    const char *n = jwxyz_nativeFontName (s->metrics_xftfont->xfont->fid, &ss);
-    sprintf (fn2, "%s %.1f", n, ss);
-  }
+  append_font_name (s->dpy, fn2, s->metrics_xftfont->xfont);
 # endif
 
   xftdraw = XftDrawCreate (s->dpy, dest, s->xgwa.visual,
@@ -2176,8 +2207,10 @@ fontglide_init (Display *dpy, Window window)
   XGetWindowAttributes (s->dpy, s->window, &s->xgwa);
 
   s->font_override = get_string_resource (dpy, "font", "Font");
-  if (s->font_override && (!*s->font_override || *s->font_override == '('))
+  if (s->font_override && (!*s->font_override || *s->font_override == '(')) {
+    free (s->font_override);
     s->font_override = 0;
+  }
 
   s->charset = get_string_resource (dpy, "fontCharset", "FontCharset");
   s->border_width = get_integer_resource (dpy, "fontBorderWidth", "Integer");
@@ -2220,7 +2253,7 @@ fontglide_init (Display *dpy, Window window)
   if (s->trails_p) s->dbuf = False;  /* don't need it in this case */
 
   {
-    const char *ss = get_string_resource (dpy, "mode", "Mode");
+    char *ss = get_string_resource (dpy, "mode", "Mode");
     if (!ss || !*ss || !strcasecmp (ss, "random"))
       s->mode = ((random() % 2) ? SCROLL : PAGE);
     else if (!strcasecmp (ss, "scroll"))
@@ -2235,6 +2268,7 @@ fontglide_init (Display *dpy, Window window)
                 "%s: `mode' must be `scroll', `page', or `random', not `%s'\n",
                  progname, ss);
       }
+    if (ss) free (ss);
   }
 
   if (s->dbuf)
@@ -2383,8 +2417,20 @@ static void
 fontglide_free (Display *dpy, Window window, void *closure)
 {
   state *s = (state *) closure;
+  int i;
+
   textclient_close (s->tc);
 
+//  if (s->b && s->b != s->window) XFreePixmap (dpy, s->b);
+//  if (s->ba && s->ba != s->b) XFreePixmap (dpy, s->ba);
+  XFreeGC (dpy, s->bg_gc);
+  if (s->charset) free (s->charset);
+  if (s->font_override) free (s->font_override);
+  for (i = 0;i < s->nsentences; i++)
+    if (s->sentences[i])
+      free_sentence (s, s->sentences[i]);
+  free (s->sentences);
+
 #ifdef DEBUG
   if (s->metrics_xftfont)
     XftFontClose (s->dpy, s->metrics_xftfont);
@@ -2397,8 +2443,6 @@ fontglide_free (Display *dpy, Window window, void *closure)
   if (s->label_gc) XFreeGC (dpy, s->label_gc);
 #endif
 
-  /* #### there's more to free here */
-
   free (s);
 }
 
