@@ -349,6 +349,13 @@ FuzzyFlakesInit(Flake *flake)
    if (flake->RandomColors == True)
       flake->RandomColors = get_boolean_resource(flake->dpy, "randomColors", "Boolean");
 
+   if (xgwa.width > 2560) {  /* Retina displays */
+     flake->Thickness *= 2;
+     flake->BorderThickness *= 2;
+     flake->Radius *= 2;
+     flake->FallingSpeed *= 2;
+   }
+
    if (flake->Delay < 0)
       flake->Delay = 0;
 
@@ -438,6 +445,10 @@ FuzzyFlakesFreeFlake(Flake *flake)
 
    if (flake->DB.bb) XFreePixmap(flake->dpy, flake->DB.bb);
    if (flake->DB.ba) XFreePixmap(flake->dpy, flake->DB.ba);
+   XFreeGC (flake->dpy, flake->GCVar);
+   if (flake->Colors.Back) free (flake->Colors.Back);
+   if (flake->Colors.Fore) free (flake->Colors.Fore);
+   if (flake->Colors.Bord) free (flake->Colors.Bord);
 }
 
 static void
@@ -600,6 +611,7 @@ fuzzyflakes_free (Display *dpy, Window window, void *closure)
 {
   Flake *flake = (Flake *) closure;
   FuzzyFlakesFreeFlake(flake);
+  free(flake->Flakes);
   free(flake);
 }
 
