@@ -19,7 +19,6 @@
 			    "*showFPS: False   \n" \
                             "*useSHM:  True    \n"
 
-# define refresh_antspotlight 0
 # define release_antspotlight 0
 #include "xlockmore.h"
 #else
@@ -552,8 +551,16 @@ static void draw_antspotlight_strip(ModeInfo *mi)
 ENTRYPOINT void reshape_antspotlight(ModeInfo * mi, int width, int height)
 {
   double h = (GLfloat) height / (GLfloat) width;  
+  int y = 0;
   int size = 2;
-  glViewport(0, 0, width, height);
+
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport(0, y, width, height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
 
@@ -697,7 +704,7 @@ ENTRYPOINT void init_antspotlight(ModeInfo *mi)
 
   antspotlightstruct *mp;
   
-  MI_INIT(mi, antspotlight, NULL);
+  MI_INIT(mi, antspotlight);
   mp = &antspotlight[MI_SCREEN(mi)];
   mp->rot = make_rotator (rot_speed, rot_speed, rot_speed, 1, 0, True);
   mp->trackball = gltrackball_init (False);
@@ -742,7 +749,7 @@ ENTRYPOINT void draw_antspotlight(ModeInfo * mi)
   /* Just keep running before the texture has come in. */
   /* if (mp->waiting_for_image_p) return; */
 
-  glXMakeCurrent(display, window, *(mp->glx_context));
+  glXMakeCurrent(display, window, *mp->glx_context);
   
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
@@ -783,9 +790,19 @@ ENTRYPOINT void change_antspotlight(ModeInfo * mi)
   if (!mp->glx_context)
 	return;
   
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(mp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
   pinit();
 }
 #endif /* !STANDALONE */
 
+ENTRYPOINT void free_antspotlight(ModeInfo * mi)
+{
+  antspotlightstruct *mp = &antspotlight[MI_SCREEN(mi)];
+  if (!mp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
+  gltrackball_free (mp->trackball);
+  free_rotator (mp->rot);
+  if (mp->screentexture) glDeleteTextures (1, &mp->screentexture);
+}
+
 XSCREENSAVER_MODULE ("AntSpotlight", antspotlight)
