@@ -1,4 +1,4 @@
-/* bouncingcow, Copyright (c) 2003-2014 Jamie Zawinski <jwz@jwz.org>
+/* bouncingcow, Copyright (c) 2003-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -16,7 +16,6 @@
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
 
-# define free_cow 0
 # define release_cow 0
 #define DEF_SPEED       "1.0"
 #define DEF_TEXTURE     "(none)"
@@ -32,7 +31,7 @@
 #include "xlockmore.h"
 #include "rotator.h"
 #include "gltrackball.h"
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 #include <ctype.h>
 
 #ifdef USE_GL /* whole file */
@@ -202,7 +201,6 @@ load_texture (ModeInfo *mi, const char *filename)
 {
   Display *dpy = mi->dpy;
   Visual *visual = mi->xgwa.visual;
-  Colormap cmap = mi->xgwa.colormap;
   char buf[1024];
   XImage *image;
 
@@ -217,16 +215,13 @@ load_texture (ModeInfo *mi, const char *filename)
       return False;
     }
 
-  image = xpm_file_to_ximage (dpy, visual, cmap, filename);
+  image = file_to_ximage (dpy, visual, filename);
   if (!image) return False;
 
   clear_gl_error();
   glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA,
                 image->width, image->height, 0,
-                GL_RGBA,
-                /* GL_UNSIGNED_BYTE, */
-                GL_UNSIGNED_INT_8_8_8_8_REV,
-                image->data);
+                GL_RGBA, GL_UNSIGNED_BYTE, image->data);
   sprintf (buf, "texture: %.100s (%dx%d)",
            filename, image->width, image->height);
   check_gl_error(buf);
@@ -309,15 +304,15 @@ init_cow (ModeInfo *mi)
               glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
               glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
               glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+              glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
+              glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
               glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
               glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
               glEnable(GL_TEXTURE_GEN_S);
               glEnable(GL_TEXTURE_GEN_T);
               glEnable(GL_TEXTURE_2D);
 
-              /* approximately line it up with ../images/earth.xpm */
+              /* approximately line it up with ../images/earth.png */
               glMatrixMode (GL_TEXTURE);
               glLoadIdentity();
               glTranslatef (0.45, 0.58, 0);
@@ -467,7 +462,7 @@ draw_cow (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -518,6 +513,25 @@ draw_cow (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_cow (ModeInfo *mi)
+{
+  cow_configuration *bp = &bps[MI_SCREEN(mi)];
+  int i;
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
+  if (bp->floaters) {
+    for (i = 0; i < bp->nfloaters; i++)
+      free_rotator (bp->floaters[i].rot);
+    free (bp->floaters);
+  }
+  for (i = 0; i < countof(all_objs); i++)
+    if (glIsList(bp->dlists[i])) glDeleteLists(bp->dlists[i], 1);
+  if (bp->trackball) gltrackball_free (bp->trackball);
+  if (bp->dlists) free (bp->dlists);
+}
+
 XSCREENSAVER_MODULE_2 ("BouncingCow", bouncingcow, cow)
 
 #endif /* USE_GL */
