@@ -82,7 +82,6 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
 							"*wireframe:	False	\n"			\
 							"*suppressRotationAnimation: True\n" \
 
-# define free_cage 0
 # define release_cage 0
 # define cage_handle_event xlockmore_no_events
 # include "xlockmore.h"		/* from the xscreensaver distribution */
@@ -95,8 +94,8 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
 #if 0
 #include "e_textures.h"
 #else
-#include "xpm-ximage.h"
-#include "../images/wood.xpm"
+#include "ximage-loader.h"
+#include "images/gen/wood_png.h"
 #endif
 
 ENTRYPOINT ModeSpecOpt cage_opts =
@@ -363,16 +362,11 @@ pinit(ModeInfo *mi)
 	check_gl_error("mipmapping");
 #else
     {
-      XImage *img = xpm_to_ximage (mi->dpy,
-                                   mi->xgwa.visual,
-                                   mi->xgwa.colormap,
-                                   wood_texture);
+      XImage *img = image_data_to_ximage (mi->dpy, mi->xgwa.visual,
+                                          wood_png, sizeof(wood_png));
 	  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA,
                     img->width, img->height, 0,
-                    GL_RGBA,
-                    /* GL_UNSIGNED_BYTE, */
-                    GL_UNSIGNED_INT_8_8_8_8_REV,
-                    img->data);
+                    GL_RGBA, GL_UNSIGNED_BYTE, img->data);
       check_gl_error("texture");
       XDestroyImage (img);
     }
@@ -423,7 +417,7 @@ draw_cage (ModeInfo * mi)
 		return;
 
     mi->polygon_count = 0;
-	glXMakeCurrent(display, window, *(cp->glx_context));
+	glXMakeCurrent(display, window, *cp->glx_context);
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -475,11 +469,17 @@ change_cage (ModeInfo * mi)
 	if (!cp->glx_context)
 		return;
 
-	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(cp->glx_context));
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *cp->glx_context);
 	pinit(mi);
 }
 #endif /* !STANDALONE */
 
+ENTRYPOINT void
+free_cage (ModeInfo * mi)
+{
+	/* nothing to do */
+}
+
 XSCREENSAVER_MODULE ("Cage", cage)
 
 #endif
