@@ -24,7 +24,6 @@
                    "*wireframe:     False         \n" \
 		   "*suppressRotationAnimation: True\n" \
 
-# define refresh_cubicgrid 0
 # define release_cubicgrid 0
 #include "xlockmore.h"
 
@@ -192,9 +191,17 @@ static void init_gl(ModeInfo *mi)
 ENTRYPOINT void reshape_cubicgrid(ModeInfo *mi, int width, int height) 
 {
   cubicgrid_conf *cp = &cubicgrid[MI_SCREEN(mi)];
+  int y = 0;
   if(!height) height = 1;
   cp->ratio = (GLfloat)width/(GLfloat)height;
-  glViewport(0, 0, (GLint) width, (GLint) height);
+
+  if (width > height * 3) {   /* tiny window: show middle */
+    height = width;
+    y = -height/2;
+    cp->ratio = (GLfloat)width/(GLfloat)height;
+  }
+
+  glViewport(0, y, (GLint) width, (GLint) height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(30.0, cp->ratio, 1.0, 100.0);
@@ -205,7 +212,7 @@ ENTRYPOINT void reshape_cubicgrid(ModeInfo *mi, int width, int height)
 ENTRYPOINT void init_cubicgrid(ModeInfo *mi) 
 {
   cubicgrid_conf *cp;
-  MI_INIT(mi, cubicgrid, NULL);
+  MI_INIT(mi, cubicgrid);
   cp = &cubicgrid[MI_SCREEN(mi)];
 
   if ((cp->glx_context = init_GL(mi)) != NULL) {
@@ -234,7 +241,7 @@ ENTRYPOINT void draw_cubicgrid(ModeInfo * mi)
   cp = &cubicgrid[MI_SCREEN(mi)];
   MI_IS_DRAWN(mi) = True;
   if (!cp->glx_context) return;
-  glXMakeCurrent(display, window, *(cp->glx_context));
+  glXMakeCurrent(display, window, *cp->glx_context);
   if (!draw_main(mi)) {
     MI_ABORT(mi);
     return;
@@ -250,11 +257,20 @@ ENTRYPOINT void change_cubicgrid(ModeInfo * mi)
 {
   cubicgrid_conf *cp = &cubicgrid[MI_SCREEN(mi)];
   if (!cp->glx_context) return;
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(cp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *cp->glx_context);
   init_gl(mi);
 }
 #endif /* !STANDALONE */
 
+ENTRYPOINT void free_cubicgrid(ModeInfo * mi) 
+{
+  cubicgrid_conf *cp = &cubicgrid[MI_SCREEN(mi)];
+  if (!cp->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *cp->glx_context);
+  gltrackball_free (cp->trackball);
+  free_rotator (cp->rot);
+  if (glIsList(cp->list)) glDeleteLists(cp->list, 1);
+}
 
 XSCREENSAVER_MODULE ("CubicGrid", cubicgrid)
 
