@@ -1,4 +1,4 @@
-/* DNA Logo, Copyright (c) 2001-2017 Jamie Zawinski <jwz@jwz.org>
+/* DNA Logo, Copyright (c) 2001-2018 Jamie Zawinski <jwz@jwz.org>
  *
  *      DNA Lounge
  *
@@ -31,17 +31,17 @@
 			"*wallFacets:	    360	    \n" \
 			"*barFacets:	    90	    \n" \
 			"*clockwise:	    False   \n" \
-			"*turns:	    0.69    \n" \
-			"*turnSpacing:	    2.2     \n" \
-			"*barSpacing:	    0.24    \n" \
-			"*wallHeight:	    0.45    \n" \
+			"*turns:	    0.72    \n" \
+			"*turnSpacing:	    2.3     \n" \
+			"*barSpacing:	    0.268   \n" \
+			"*wallHeight:	    0.42    \n" \
 			"*wallThickness:    0.12    \n" \
 			"*barThickness:     0.058   \n" \
 			"*wallTaper:	    0.95    \n" \
-			"*gasketSize:	    1.88    \n" \
+			"*gasketSize:	    2.0     \n" \
 			"*gasketDepth:	    0.15    \n" \
 			"*gasketThickness:  0.4	    \n" \
-			"*frameSize:	    1.20    \n" \
+			"*frameSize:	    1.28    \n" \
 			"*frameDepth:	    0.01    \n" \
 			"*frameThickness:   0.03    \n" \
 			"*triangleSize:	    0.045   \n" \
@@ -67,7 +67,6 @@
 #  define CWFONT "-*-helvetica-medium-r-normal-*-*-240-*-*-*-*-*-*"
 # endif
 
-# define free_logo 0
 # define release_logo 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -167,6 +166,11 @@ typedef struct {
   texture_font_data *font;
 # endif
 
+# ifdef DEBUG
+  GLfloat persp_off, pos_off;
+  texture_font_data *label_font;
+# endif
+
   GLfloat speed;
   glyph_mode mode;
   glyph_mode anim_state;
@@ -560,7 +564,7 @@ make_helix (logo_configuration *dc, int facetted, int wire)
   th  = 0;
   x1  = 1;
   y1  = 0;
-  x1b = 1 - dc->wall_thickness;
+  x1b = 1;
   y1b = 0;
 
   z1 = -(dc->turn_spacing * dc->turns / 2);
@@ -591,13 +595,13 @@ make_helix (logo_configuration *dc, int facetted, int wire)
 
   while (th + th_inc <= max_th)
     {
+      GLfloat thick = dc->wall_thickness;
+
       th += th_inc;
 
       x2 = cos (th);
       y2 = sin (th);
       z2 = z1 + z_inc;
-      x2b = x2 * (1 - dc->wall_thickness);
-      y2b = y2 * (1 - dc->wall_thickness);
 
       h2 = h1;
       h2off = h1off;
@@ -613,6 +617,11 @@ make_helix (logo_configuration *dc, int facetted, int wire)
                 h2off = h2 - dc->wall_height/2;
               else
                 h2off = dc->wall_height/2 - h2;
+
+              if (th + th_inc <= 0)
+                thick = 0;
+              else
+              thick *= cos (M_PI / 2 * (1 - (th / dc->wall_taper)));
             }
           else if (th >= max_th - dc->wall_taper)
             {
@@ -626,9 +635,17 @@ make_helix (logo_configuration *dc, int facetted, int wire)
                 h2off = dc->wall_height/2 - h2;
               else
                 h2off = h2 - dc->wall_height/2;
+
+              if (th + th_inc > max_th)
+                thick = 0;
+              else
+                thick *= cos(M_PI / 2 * (1 - ((max_th - th)/dc->wall_taper)));
             }
         }
 
+      x2b = x2 * (1 - thick);
+      y2b = y2 * (1 - thick);
+
       /* outer face
        */
       glFrontFace(GL_CW);
@@ -742,6 +759,11 @@ make_ladder (logo_configuration *dc, int facetted, int wire)
   th = (max_th * pad_ratio/2);
   z  = -(max_z / 2) + (max_z * pad_ratio/2);
 
+  /* ##### WHYYYYYY */
+  /* The image is not reflected across line y = -x and I don't know why. */
+  th += M_PI * -0.035;
+  z -= 0.08;
+
   if (!dc->clockwise)
     z = -z, z_inc = -z_inc;
 
@@ -2730,25 +2752,33 @@ draw_codeword_path (ModeInfo *mi)
 
 #endif /* CW */
 
-
-/* Window management, etc
- */
+
 ENTRYPOINT void
 reshape_logo (ModeInfo *mi, int width, int height)
 {
+# ifdef DEBUG
+  logo_configuration *dc = &dcs[MI_SCREEN(mi)];
+# endif
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  GLfloat persp = 64;  /* 30 */
+  GLfloat pos   = 13;  /* 30 */
+
+# ifdef DEBUG
+  persp += dc->persp_off;
+  pos += dc->pos_off;
+# endif
 
   glViewport (0, 0, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  gluPerspective (30.0, 1/h, 1.0, 100.0);
+  gluPerspective (persp, 1/h, 1, 100);
 
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-  gluLookAt( 0.0, 0.0, 30.0,
-             0.0, 0.0, 0.0,
-             0.0, 1.0, 0.0);
+  gluLookAt( 0, 0, pos,
+             0, 0, 0,
+             0, 1, 0);
 
 # ifdef HAVE_MOBILE	/* Keep it the same relative size when rotated. */
   {
@@ -2886,6 +2916,10 @@ init_logo (ModeInfo *mi)
     dc->font = load_texture_font (MI_DISPLAY(mi), "cwFont");
 # endif
 
+# ifdef DEBUG
+    dc->label_font = load_texture_font (MI_DISPLAY(mi), "fpsFont");
+# endif
+
   {
     XColor xcolor;
 
@@ -2909,6 +2943,7 @@ init_logo (ModeInfo *mi)
     dc->color[2] = xcolor.blue  / 65535.0;
     dc->color[3] = 1.0;
 
+    free (color_name);
     color_name = get_string_resource (mi->dpy, "cwForeground", "Foreground");
     for (s2 = color_name + strlen(color_name) - 1; s2 > color_name; s2--)
       if (*s2 == ' ' || *s2 == '\t')
@@ -2921,6 +2956,7 @@ init_logo (ModeInfo *mi)
         fprintf (stderr, "%s: can't parse color %s\n", progname, color_name);
         exit (1);
       }
+    free (color_name);
 
 # ifdef CW
     dc->codeword_color[0] = xcolor.red   / 65535.0;
@@ -2940,6 +2976,7 @@ init_logo (ModeInfo *mi)
         fprintf (stderr, "%s: can't parse color %s\n", progname, color_name);
         exit (1);
       }
+    free (color_name);
 
     dc->codeword_bg[0] = xcolor.red   / 65535.0;
     dc->codeword_bg[1] = xcolor.green / 65535.0;
@@ -3127,6 +3164,22 @@ logo_handle_event (ModeInfo *mi, XEvent *event)
       KeySym keysym;
       char c = 0;
       XLookupString (&event->xkey, &c, 1, &keysym, 0);
+
+# ifdef DEBUG
+      {
+        GLfloat step = 0.1;
+        if      (c == 'a') dc->persp_off += step;
+        else if (c == 'z') dc->persp_off -= step;
+        else if (c == 's') dc->pos_off   += step;
+        else if (c == 'x') dc->pos_off   -= step;
+        else return False;
+
+        /* dc->pos_off = -dc->persp_off; */
+        reshape_logo (mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+        return True;
+      }
+# endif
+
       if (c == ' ' || c == '\t')
         {
           switch (dc->anim_state) {
@@ -3238,7 +3291,7 @@ draw_logo (ModeInfo *mi)
     return;
 
   mi->polygon_count = 0;
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(dc->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *dc->glx_context);
 
   if (!wire &&
       dc->wire_overlay == 0 &&
@@ -3247,6 +3300,7 @@ draw_logo (ModeInfo *mi)
                         (random() % 200) +
                         (random() % 200));
       
+# ifndef DEBUG
   tick_spinner (mi, &dc->gasket_spinnerx);
   tick_spinner (mi, &dc->gasket_spinnery);
   tick_spinner (mi, &dc->gasket_spinnerz);
@@ -3257,6 +3311,7 @@ draw_logo (ModeInfo *mi)
   tick_spinner (mi, &dc->scene_spinnery);
   tick_spinner (mi, &dc->frame_spinner);
   link_spinners (mi, &dc->scene_spinnerx, &dc->scene_spinnery);
+# endif /* DEBUG */
 
   switch (dc->anim_state)
     {
@@ -3364,6 +3419,11 @@ draw_logo (ModeInfo *mi)
       break;
     }
 
+# ifdef DEBUG
+  dc->anim_state = HELIX;
+  dc->wire_overlay = 0;
+# endif
+
   pizza_p = (dc->anim_state == PIZZA ||
              dc->anim_state == PIZZA_IN ||
              dc->anim_state == PIZZA_OUT);
@@ -3559,12 +3619,52 @@ draw_logo (ModeInfo *mi)
   if (dc->wire_overlay > 0)
     dc->wire_overlay--;
 
+# ifdef DEBUG
+  {
+    char s[1024];
+    sprintf (s, "a/z, s/x; per = %0.2f pos = %0.2f",
+             dc->persp_off, dc->pos_off);
+    glColor3f (1,1,1);
+    print_texture_label (dpy, dc->label_font, MI_WIDTH(mi), MI_HEIGHT(mi),
+                         1, s);
+  }
+# endif
+
   if (mi->fps_p) do_fps (mi);
   glFinish();
 
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_logo (ModeInfo *mi)
+{
+  logo_configuration *dc = &dcs[MI_SCREEN(mi)];
+  if (!dc->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *dc->glx_context);
+  if (dc->trackball) gltrackball_free (dc->trackball);
+# ifdef CW
+  if (dc->codeword_text) free (dc->codeword_text);
+  if (dc->codeword_text_out) free (dc->codeword_text_out);
+  if (dc->scene_rot) free_rotator (dc->scene_rot);
+  if (dc->font) free_texture_font (dc->font);
+# endif
+# ifdef DEBUG
+  if (dc->label_font) free_texture_font (dc->label_font);
+# endif
+  glDeleteLists(dc->helix_list, 1);
+  glDeleteLists(dc->helix_list_wire, 1);
+  glDeleteLists(dc->helix_list_facetted, 1);
+  glDeleteLists(dc->pizza_list, 1);
+  glDeleteLists(dc->pizza_list_wire, 1);
+  glDeleteLists(dc->pizza_list_facetted, 1);
+  glDeleteLists(dc->gasket_list, 1);
+  glDeleteLists(dc->gasket_list_wire, 1);
+  glDeleteLists(dc->frame_list, 1);
+  glDeleteLists(dc->frame_list_wire, 1);
+}
+
 XSCREENSAVER_MODULE_2 ("DNALogo", dnalogo, logo)
 
 #endif /* USE_GL */
