@@ -26,9 +26,8 @@
 		 ".foreground: " DEF_COLOR "\n" \
 		 "*program: xscreensaver-text --cols 0"  /* don't wrap */
 
-# define refresh_fliptext 0
 # define release_fliptext 0
-# define fliptext_handle_event 0
+# define fliptext_handle_event xlockmore_no_events
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
@@ -768,8 +767,15 @@ reshape_fliptext (ModeInfo *mi, int width, int height)
 {
   fliptext_configuration *sc = &scs[MI_SCREEN(mi)];
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -788,17 +794,15 @@ reshape_fliptext (ModeInfo *mi, int width, int height)
 }
 
 
-static void free_fliptext (ModeInfo *mi);
-
-
 ENTRYPOINT void 
 init_fliptext (ModeInfo *mi)
 {
   int wire = MI_IS_WIREFRAME(mi);
+  char *s;
 
   fliptext_configuration *sc;
 
-  MI_INIT(mi, scs, free_fliptext);
+  MI_INIT(mi, scs);
 
   sc = &scs[MI_SCREEN(mi)];
   sc->lines = (line **) calloc (max_lines+1, sizeof(char *));
@@ -894,9 +898,9 @@ init_fliptext (ModeInfo *mi)
   if (min_lines > max_lines - 3) min_lines = max_lines - 4;
   if (min_lines < 1) min_lines = 1;
 
-  parse_color (mi, "foreground",
-               get_string_resource(mi->dpy, "foreground", "Foreground"),
-               sc->color);
+  s = get_string_resource(mi->dpy, "foreground", "Foreground");
+  parse_color (mi, "foreground", s, sc->color);
+  if (s) free (s);
 
   sc->top_margin = (sc->char_width * 100);
   sc->bottom_margin = -sc->top_margin;
@@ -916,7 +920,7 @@ draw_fliptext (ModeInfo *mi)
   if (!sc->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sc->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *sc->glx_context);
 
 #if 0
   if (XtAppPending (app) & (XtIMTimer|XtIMAlternateInput))
@@ -982,15 +986,22 @@ draw_fliptext (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
-static void
+ENTRYPOINT void
 free_fliptext (ModeInfo *mi)
 {
   fliptext_configuration *sc = &scs[MI_SCREEN(mi)];
+  int i;
+  if (!sc->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *sc->glx_context);
   if (sc->tc)
     textclient_close (sc->tc);
-  free(sc->lines);
-
-  /* #### there's more to free here */
+  if (sc->texfont) free_texture_font (sc->texfont);
+  for (i = 0; i < sc->nlines; i++)
+    if (sc->lines[i]) {
+      if (sc->lines[i]->text) free (sc->lines[i]->text);
+      free (sc->lines[i]);
+    }
+  if(sc->lines) free(sc->lines);
 }
 
 XSCREENSAVER_MODULE ("FlipText", fliptext)
