@@ -18,7 +18,6 @@
 			"*wireframe:    False       \n" \
 			"*suppressRotationAnimation: True\n" \
 
-# define refresh_gears 0
 # define release_gears 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -86,8 +85,15 @@ ENTRYPOINT void
 reshape_gears (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -698,7 +704,7 @@ init_gears (ModeInfo *mi)
   int wire = MI_IS_WIREFRAME(mi);
   int i;
 
-  MI_INIT (mi, bps, NULL);
+  MI_INIT (mi, bps);
 
   bp = &bps[MI_SCREEN(mi)];
 
@@ -820,7 +826,7 @@ draw_gears (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   glShadeModel(GL_SMOOTH);
 
@@ -926,6 +932,24 @@ gears_handle_event (ModeInfo *mi, XEvent *event)
   return False;
 }
 
+
+ENTRYPOINT void
+free_gears (ModeInfo *mi)
+{
+  gears_configuration *bp = &bps[MI_SCREEN(mi)];
+  int i;
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
+  if (bp->rot) free_rotator (bp->rot);
+  if (bp->trackball) gltrackball_free (bp->trackball);
+  for (i = 0; i < bp->ngears; i++)
+    if (bp->gears[i])
+      free_gear (bp->gears[i]);
+  if (glIsList(bp->armature_dlist))
+    glDeleteLists (bp->armature_dlist, 1);
+
+}
+
 XSCREENSAVER_MODULE ("Gears", gears)
 
 #endif /* USE_GL */
