@@ -14,7 +14,6 @@
 			"*showFPS:      False       \n" \
 			"*suppressRotationAnimation: True\n" \
 
-# define refresh_geodesic 0
 # define release_geodesic 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -456,8 +455,15 @@ ENTRYPOINT void
 reshape_geodesic (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -488,7 +494,7 @@ init_geodesic (ModeInfo *mi)
   geodesic_configuration *bp;
   int wire = MI_IS_WIREFRAME(mi);
 
-  MI_INIT (mi, bps, NULL);
+  MI_INIT (mi, bps);
 
   bp = &bps[MI_SCREEN(mi)];
 
@@ -618,7 +624,7 @@ draw_geodesic (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   if (! wire)
     glShadeModel(GL_SMOOTH);
@@ -804,6 +810,18 @@ draw_geodesic (ModeInfo *mi)
     }
 }
 
+
+ENTRYPOINT void
+free_geodesic (ModeInfo *mi)
+{
+  geodesic_configuration *bp = &bps[MI_SCREEN(mi)];
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
+  if (bp->trackball) gltrackball_free (bp->trackball);
+  if (bp->rot) free_rotator (bp->rot);
+  if (bp->colors) free (bp->colors);
+}
+
 XSCREENSAVER_MODULE ("Geodesic", geodesic)
 
 #endif /* USE_GL */
