@@ -18,7 +18,10 @@
 
 #include <stdlib.h>
 
-#ifndef HAVE_COCOA
+#ifdef HAVE_COCOA
+#elif defined(HAVE_ANDROID)
+# include <GLES/gl.h>
+#else /* real X11 */
 # include <GL/gl.h>
 #endif
 
