@@ -1,4 +1,4 @@
-/* glmatrix, Copyright (c) 2003, 2004 Jamie Zawinski <jwz@jwz.org>
+/* glmatrix, Copyright (c) 2003-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -20,7 +20,6 @@
 			"*showFPS:      False         \n" \
 			"*wireframe:    False         \n" \
 
-# define free_matrix 0
 # define release_matrix 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -29,14 +28,9 @@
 #define BELLRAND(n) ((frand((n)) + frand((n)) + frand((n))) / 3)
 
 #include "xlockmore.h"
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 
-#ifdef __GNUC__
-  __extension__  /* don't warn about "string length is greater than the length
-                    ISO C89 compilers are required to support" when including
-                    the following XPM file... */
-#endif
-#include "../images/matrix3.xpm"
+#include "images/gen/matrix3_png.h"
 
 #ifdef USE_GL /* whole file */
 
@@ -703,11 +697,11 @@ load_textures (ModeInfo *mi, Bool flip_p)
   int cw, ch;
   int orig_w, orig_h;
 
-  /* The Matrix XPM is 512x598 -- but GL texture sizes must be powers of 2.
+  /* The Matrix image is 512x598 -- but GL texture sizes must be powers of 2.
      So we waste some padding rows to round up.
    */
-  xi = xpm_to_ximage (mi->dpy, mi->xgwa.visual, mi->xgwa.colormap,
-                      matrix3_xpm);
+  xi = image_data_to_ximage (mi->dpy, mi->xgwa.visual, 
+                             matrix3_png, sizeof(matrix3_png));
   orig_w = xi->width;
   orig_h = xi->height;
   mp->real_char_rows = CHAR_ROWS;
@@ -794,7 +788,7 @@ load_textures (ModeInfo *mi, Bool flip_p)
   glBindTexture (GL_TEXTURE_2D, mp->texture);
   check_gl_error ("texture init");
   glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, xi->width, xi->height, 0, GL_RGBA,
-                GL_UNSIGNED_INT_8_8_8_8_REV, xi->data);
+                GL_UNSIGNED_BYTE, xi->data);
   {
     char buf[255];
     sprintf (buf, "creating %dx%d texture:", xi->width, xi->height);
@@ -991,7 +985,7 @@ draw_matrix (ModeInfo *mi)
   if (!mp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(mp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -1068,6 +1062,17 @@ draw_matrix (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_matrix (ModeInfo *mi)
+{
+  matrix_configuration *mp = &mps[MI_SCREEN(mi)];
+  if (!mp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *mp->glx_context);
+  if (mp->strips) free (mp->strips);
+  if (mp->texture) glDeleteTextures (1, &mp->texture);
+}
+
 XSCREENSAVER_MODULE_2 ("GLMatrix", glmatrix, matrix)
 
 #endif /* USE_GL */
