@@ -89,7 +89,6 @@ static const char sccsid[] = "@(#)hypertorus.c  1.2 05/09/28 xlockmore";
                             "*showFPS:    False \n" \
 			    "*suppressRotationAnimation: True\n" \
 
-# define refresh_hypertorus 0
 # define release_hypertorus 0
 # include "xlockmore.h"         /* from the xscreensaver distribution */
 #else  /* !STANDALONE */
@@ -744,11 +743,19 @@ static void display_hypertorus(ModeInfo *mi)
 ENTRYPOINT void reshape_hypertorus(ModeInfo *mi, int width, int height)
 {
   hypertorusstruct *hp = &hyper[MI_SCREEN(mi)];
+  double h = (GLfloat) height / (GLfloat) width;  
+  int y = 0;
+
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
 
   hp->WindW = (GLint)width;
   hp->WindH = (GLint)height;
-  glViewport(0,0,width,height);
-  hp->aspect = (GLfloat)width/(GLfloat)height;
+  glViewport(0,y,width,height);
+  hp->aspect = h;
 }
 
 
@@ -830,7 +837,7 @@ ENTRYPOINT void init_hypertorus(ModeInfo *mi)
 {
   hypertorusstruct *hp;
 
-  MI_INIT(mi, hyper, NULL);
+  MI_INIT(mi, hyper);
   hp = &hyper[MI_SCREEN(mi)];
 
   
@@ -974,7 +981,7 @@ ENTRYPOINT void draw_hypertorus(ModeInfo *mi)
   if (!hp->glx_context)
     return;
 
-  glXMakeCurrent(display,window,*(hp->glx_context));
+  glXMakeCurrent(display, window, *hp->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();
@@ -998,11 +1005,20 @@ ENTRYPOINT void change_hypertorus(ModeInfo *mi)
   if (!hp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi),MI_WINDOW(mi),*(hp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *hp->glx_context);
   init(mi);
 }
 #endif /* !STANDALONE */
 
+ENTRYPOINT void free_hypertorus(ModeInfo *mi)
+{
+  hypertorusstruct *hp = &hyper[MI_SCREEN(mi)];
+  if (!hp->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *hp->glx_context);
+  gltrackball_free (hp->trackballs[0]);
+  gltrackball_free (hp->trackballs[1]);
+}
+
 XSCREENSAVER_MODULE ("Hypertorus", hypertorus)
 
 #endif /* USE_GL */
