@@ -44,7 +44,6 @@
                  "*chooseRandomImages:  True  \n"                           \
 		 "*suppressRotationAnimation: True\n"                       \
 
-# define refresh_mirrorblob 0
 # define release_mirrorblob 0
 # include "xlockmore.h"
 #else /* !STANDALONE */
@@ -179,7 +178,7 @@ ENTRYPOINT ModeSpecOpt mirrorblob_opts = {countof(opts), opts, countof(vars), va
 #ifdef USE_MODULES
 ModStruct   mirrorblob_description =
 {"mirrorblob", "init_mirrorblob", "draw_mirrorblob", NULL,
- "draw_mirrorblob", "init_mirrorblob", "handle_event", &mirrorblob_opts,
+ "draw_mirrorblob", "init_mirrorblob", "free_mirrorblob", &mirrorblob_opts,
  1000, 1, 2, 1, 4, 1.0, "",
  "OpenGL mirrorblob", 0, NULL};
 #endif
@@ -1730,7 +1729,7 @@ draw_mirrorblob(ModeInfo * mi)
   if (gp->waiting_for_image_p && gp->first_image_p)
     return;
 
-  glXMakeCurrent(display, window, *(gp->glx_context));
+  glXMakeCurrent(display, window, *gp->glx_context);
   draw_scene(mi);
   if (mi->fps_p) do_fps (mi);
   glFinish();
@@ -1786,8 +1785,6 @@ mirrorblob_handle_event (ModeInfo * mi, XEvent * event)
   return False;
 }
 
-static void free_mirrorblob(ModeInfo * mi);
-
 /******************************************************************************
  *
  * XMirrorblob initialise entry
@@ -1799,7 +1796,7 @@ init_mirrorblob(ModeInfo * mi)
 
   mirrorblobstruct *gp;
 
-  MI_INIT(mi, Mirrorblob, free_mirrorblob);
+  MI_INIT(mi, Mirrorblob);
   gp = &Mirrorblob[screen];
 
   gp->window = MI_WINDOW(mi);
@@ -1825,18 +1822,28 @@ init_mirrorblob(ModeInfo * mi)
  *
  * XMirrorblob cleanup entry
  */
-static void
+ENTRYPOINT void
 free_mirrorblob(ModeInfo * mi)
 {
   mirrorblobstruct *gp = &Mirrorblob[MI_SCREEN(mi)];
+  int i;
+
+  if (!gp->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *gp->glx_context);
+
   if (gp->nodes) free(gp->nodes);
   if (gp->faces) free(gp->faces);
   if (gp->bump_data) free(gp->bump_data);
   if (gp->colours) free(gp->colours);
   if (gp->tex_coords) free(gp->tex_coords);
   if (gp->dots) free(gp->dots);
+  if (gp->normals) free(gp->normals);
   if (gp->wall_shape) free(gp->wall_shape);
   if (gp->bump_shape) free(gp->bump_shape);
+  if (gp->trackball) gltrackball_free (gp->trackball);
+
+  for (i = 0; i < NUM_TEXTURES; i++)
+    if (gp->textures[i]) glDeleteTextures(1, &gp->textures[i]);
 }
 
 XSCREENSAVER_MODULE ("MirrorBlob", mirrorblob)
