@@ -1,4 +1,4 @@
-/* photopile, Copyright (c) 2008-2015 Jens Kilian <jjk@acm.org>
+/* photopile, Copyright (c) 2008-2018 Jens Kilian <jjk@acm.org>
  * Based on carousel, Copyright (c) 2005-2008 Jamie Zawinski <jwz@jwz.org>
  * Loads a sequence of images and shuffles them into a pile.
  *
@@ -13,8 +13,10 @@
 
 #if defined(HAVE_COCOA) || defined(HAVE_ANDROID)
 # define DEF_FONT "OCR A Std 48, Lucida Console 48, Monaco 48"
-#else
+#elif 0  /* real X11, XQueryFont() */
 # define DEF_FONT "-*-helvetica-bold-r-normal-*-*-480-*-*-*-*-*-*"
+#else    /* real X11, load_font_retry() */
+# define DEF_FONT "-*-ocr a std-medium-r-*-*-*-480-*-*-m-*-*-*"
 #endif
 
 #define DEFAULTS  "*count:           7         \n" \
@@ -29,7 +31,6 @@
                   "*chooseRandomImages:  True  \n" \
 		  "*suppressRotationAnimation: True\n" \
 
-# define free_photopile 0
 # define release_photopile 0
 # define photopile_handle_event xlockmore_no_events
 
@@ -469,6 +470,7 @@ hack_resources (Display *dpy)
   value.addr = buf2;
   value.size = strlen(buf2);
   XrmPutResource (&db, buf1, "String", &value);
+  free (val);
 # endif /* !HAVE_JWXYZ */
 }
 
@@ -662,6 +664,7 @@ draw_image (ModeInfo *mi, int i, GLfloat t, GLfloat s, GLfloat z)
 
 # if defined(HAVE_COCOA)
       scale /= 2;
+      if (MI_WIDTH(mi) > 2560) scale /= 2;  /* Retina displays */
 # endif
 
 # if defined(HAVE_MOBILE)
@@ -727,7 +730,7 @@ draw_photopile (ModeInfo *mi)
   if (!ss->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(ss->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *ss->glx_context);
 
   if (ss->mode == EARLY)
     if (loading_initial_image (mi))
@@ -825,6 +828,24 @@ draw_photopile (ModeInfo *mi)
   glXSwapBuffers (MI_DISPLAY (mi), MI_WINDOW(mi));
 }
 
+
+ENTRYPOINT void
+free_photopile (ModeInfo *mi)
+{
+  photopile_state *ss = &sss[MI_SCREEN(mi)];
+  if (!ss->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *ss->glx_context);
+  if (ss->frames) {
+    int i;
+    for (i = 0; i < MI_COUNT(mi); i++) {
+      if (ss->frames[i].title) free (ss->frames[i].title);
+      if (ss->frames[i].texid) glDeleteTextures(1, &ss->frames[i].texid);
+    }
+  }
+  if (ss->shadow) glDeleteTextures(1, &ss->shadow);
+  if (ss->texfont) free_texture_font (ss->texfont);
+}
+
 XSCREENSAVER_MODULE ("Photopile", photopile)
 
 #endif /* USE_GL */
