@@ -68,7 +68,6 @@ static const char sccsid[] = "@(#)pipes.c	4.07 97/11/24 xlockmore";
 	               	"*wireframe:    False   \n"		    \
 					"*suppressRotationAnimation: True\n" \
 
-# define refresh_pipes 0
 # define release_pipes 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 #else  /* !STANDALONE */
@@ -135,7 +134,7 @@ ENTRYPOINT ModeSpecOpt pipes_opts =
 ModStruct   pipes_description =
 {"pipes", "init_pipes", "draw_pipes", NULL,
  "draw_pipes",
- "change_pipes", NULL, &pipes_opts,
+ "change_pipes", "free_pipes", &pipes_opts,
  1000, 2, 5, 500, 4, 1.0, "",
  "Shows a selfbuilding pipe system", 0, NULL};
 
@@ -184,7 +183,7 @@ typedef struct {
 	GLXContext *glx_context;
 
     Bool button_down_p;
-    trackball_state *trackball;
+   trackball_state *trackball;
     GLuint *dlists, *poly_counts;
     int dlist_count, dlist_size;
     int system_index, system_size;
@@ -656,12 +655,20 @@ pinit(ModeInfo * mi, int zera)
 ENTRYPOINT void
 reshape_pipes(ModeInfo * mi, int width, int height)
 {
-	glViewport(0, 0, width, (GLint) height);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	/*glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0); */
-	gluPerspective(65.0, (GLfloat) width / (GLfloat) height, 0.1, 20.0);
-	glMatrixMode(GL_MODELVIEW);
+  double h = (GLfloat) height / (GLfloat) width;  
+  int y = 0;
+
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+  glViewport(0, y, width, (GLint) height);
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  /*glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0); */
+  gluPerspective(65.0, 1/h, 0.1, 20.0);
+  glMatrixMode(GL_MODELVIEW);
 
   glClear(GL_COLOR_BUFFER_BIT);
 }
@@ -689,15 +696,13 @@ pipes_handle_event (ModeInfo *mi, XEvent *event)
 static void generate_system (ModeInfo *);
 
 
-static void free_pipes (ModeInfo *);
-
 ENTRYPOINT void
 init_pipes (ModeInfo * mi)
 {
 	int         screen = MI_SCREEN(mi);
 	pipesstruct *pp;
 
-	MI_INIT (mi, pipes, free_pipes);
+	MI_INIT (mi, pipes);
 	pp = &pipes[screen];
 
 	pp->window = MI_WINDOW(mi);
@@ -1074,7 +1079,7 @@ draw_pipes (ModeInfo * mi)
 	if (!pp->glx_context)
 		return;
 
-	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(pp->glx_context));
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *pp->glx_context);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 	glColor3f(1.0, 1.0, 1.0);
@@ -1161,53 +1166,52 @@ change_pipes (ModeInfo * mi)
 	if (!pp->glx_context)
 		return;
 
-	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(pp->glx_context));
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *pp->glx_context);
 	pinit(mi, 1);
 }
 #endif /* !STANDALONE */
 
 
-static void
+ENTRYPOINT void
 free_pipes (ModeInfo * mi)
 {
 	pipesstruct *pp = &pipes[MI_SCREEN(mi)];
 
-	if (pp->glx_context) {
+    if (!pp->glx_context) return;
+	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *pp->glx_context);
 
-		/* Display lists MUST be freed while their glXContext is current. */
-		glXMakeCurrent(MI_DISPLAY(mi), pp->window, *(pp->glx_context));
+    if (pp->trackball) gltrackball_free (pp->trackball);
 
-		if (pp->valve)
-			glDeleteLists(pp->valve, 1);
-		if (pp->bolts)
-			glDeleteLists(pp->bolts, 1);
-		if (pp->betweenbolts)
-			glDeleteLists(pp->betweenbolts, 1);
+    if (pp->valve)
+        glDeleteLists(pp->valve, 1);
+    if (pp->bolts)
+        glDeleteLists(pp->bolts, 1);
+    if (pp->betweenbolts)
+        glDeleteLists(pp->betweenbolts, 1);
 
-		if (pp->elbowbolts)
-			glDeleteLists(pp->elbowbolts, 1);
-		if (pp->elbowcoins)
-			glDeleteLists(pp->elbowcoins, 1);
+    if (pp->elbowbolts)
+        glDeleteLists(pp->elbowbolts, 1);
+    if (pp->elbowcoins)
+        glDeleteLists(pp->elbowcoins, 1);
 
-		if (pp->guagehead)
-			glDeleteLists(pp->guagehead, 1);
-		if (pp->guageface)
-			glDeleteLists(pp->guageface, 1);
-		if (pp->guagedial)
-			glDeleteLists(pp->guagedial, 1);
-		if (pp->guageconnector)
-			glDeleteLists(pp->guageconnector, 1);
-		if (pp->teapot)
-			glDeleteLists(pp->teapot, 1);
-        if (pp->dlists)
-          {
-            int i;
-            for (i = 0; i < pp->dlist_count; i++)
-              glDeleteLists (pp->dlists[i], 1);
-            free (pp->dlists);
-            free (pp->poly_counts);
-          }
-	}
+    if (pp->guagehead)
+        glDeleteLists(pp->guagehead, 1);
+    if (pp->guageface)
+        glDeleteLists(pp->guageface, 1);
+    if (pp->guagedial)
+        glDeleteLists(pp->guagedial, 1);
+    if (pp->guageconnector)
+        glDeleteLists(pp->guageconnector, 1);
+    if (pp->teapot)
+        glDeleteLists(pp->teapot, 1);
+    if (pp->dlists)
+      {
+        int i;
+        for (i = 0; i < pp->dlist_count; i++)
+          glDeleteLists (pp->dlists[i], 1);
+        free (pp->dlists);
+        free (pp->poly_counts);
+      }
 }
 
 XSCREENSAVER_MODULE ("Pipes", pipes)
