@@ -61,7 +61,7 @@
 # endif /* VMS */
 #endif
 
-#include "xpm-ximage.h"
+#include "ximage-loader.h"
 
 /* Functions for loading and storing textures */
 
@@ -236,8 +236,8 @@ static void Create_Texture(ModeInfo *mi, const char *filename)
     }
   else
     {
-      XImage *ximage = xpm_file_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                           MI_COLORMAP (mi), filename);
+      XImage *ximage = file_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                       filename);
       if (! ximage)
         goto BUILTIN;
       image  = (GLubyte *) ximage->data;
@@ -467,7 +467,7 @@ ENTRYPOINT void draw_pulsar(ModeInfo * mi)
   if (!gp->glx_context)
 	return;
 
-  glXMakeCurrent(display, window, *(gp->glx_context));
+  glXMakeCurrent(display, window, *gp->glx_context);
   drawScene(mi);
   if (mi->fps_p) do_fps (mi);
   glXSwapBuffers(display, window);
@@ -508,7 +508,10 @@ init_pulsar(ModeInfo * mi)
 ENTRYPOINT void free_pulsar(ModeInfo * mi)
 {
   pulsarstruct *gp = &Pulsar[MI_SCREEN(mi)];
+  if (!gp->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *gp->glx_context);
   free(gp->quads);
+  if (glIsList(gp->quad_list)) glDeleteLists(gp->quad_list, 1);
 }
 #endif
 
