@@ -24,7 +24,6 @@
 			"*wireframe:    False       \n" \
 			"*suppressRotationAnimation: True\n" \
 
-# define free_quasicrystal 0
 # define release_quasicrystal 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -66,7 +65,9 @@ static XrmOptionDescRec opts[] = {
   { "-wander",       ".wander",    XrmoptionNoArg, "True"  },
   { "+wander",       ".wander",    XrmoptionNoArg, "False" },
   { "-symmetry",     ".symmetric", XrmoptionNoArg, "True"   },
+  { "-symmetric",    ".symmetric", XrmoptionNoArg, "True"   },
   { "-no-symmetry",  ".symmetric", XrmoptionNoArg, "False"  },
+  { "-nonsymmetric", ".symmetric", XrmoptionNoArg, "False"  },
   { "-speed",        ".speed",     XrmoptionSepArg, 0 },
   { "-contrast",     ".contrast",  XrmoptionSepArg, 0 },
 };
@@ -227,7 +228,7 @@ init_quasicrystal (ModeInfo *mi)
     }
 
   bp->symmetric_p =
-    get_boolean_resource (MI_DISPLAY (mi), "symmetry", "Symmetry");
+    get_boolean_resource (MI_DISPLAY (mi), "symmetric", "Symmetric");
 
   bp->contrast = get_float_resource (MI_DISPLAY (mi), "contrast", "Contrast");
   if (bp->contrast < 0 || bp->contrast > 100) 
@@ -306,7 +307,7 @@ draw_quasicrystal (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -471,6 +472,24 @@ draw_quasicrystal (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_quasicrystal (ModeInfo *mi)
+{
+  quasicrystal_configuration *bp = &bps[MI_SCREEN(mi)];
+  int i;
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
+  for (i = 0; i < bp->count; i++) {
+    free_rotator (bp->planes[i].rot);
+    free_rotator (bp->planes[i].rot2);
+    if (bp->planes[i].texid) glDeleteTextures (1, &bp->planes[i].texid);
+  }    
+  if (bp->planes) free (bp->planes);
+  if (bp->colors) free (bp->colors);
+
+}
+
 XSCREENSAVER_MODULE ("QuasiCrystal", quasicrystal)
 
 #endif /* USE_GL */
