@@ -16,7 +16,6 @@
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
 
-# define refresh_hoop 0
 # define release_hoop 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -591,8 +590,15 @@ ENTRYPOINT void
 reshape_hoop (ModeInfo *mi, int width, int height)
 {
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glViewport (0, 0, (GLint) width, (GLint) height);
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
+
+  glViewport (0, y, (GLint) width, (GLint) height);
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -649,7 +655,7 @@ init_hoop (ModeInfo *mi)
   hoop_configuration *bp;
   int wire = MI_IS_WIREFRAME(mi);
 
-  MI_INIT (mi, bps, NULL);
+  MI_INIT (mi, bps);
 
   bp = &bps[MI_SCREEN(mi)];
 
@@ -708,7 +714,7 @@ draw_hoop (ModeInfo *mi)
   if (!bp->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(bp->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -755,6 +761,31 @@ draw_hoop (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_hoop (ModeInfo *mi)
+{
+  hoop_configuration *bp = &bps[MI_SCREEN(mi)];
+
+  if (!bp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *bp->glx_context);
+
+  if (bp->trackball) gltrackball_free (bp->trackball);
+  if (bp->rot) free_rotator (bp->rot);
+  if (bp->lights) free (bp->lights);
+  while (bp->trail) {
+    afterimage *n = bp->trail->next;
+    free (bp->trail);
+    bp->trail = n;
+  }
+  while (bp->oscillators) {
+    oscillator *n = bp->oscillators->next;
+    free (bp->oscillators);
+    bp->oscillators = n;
+  }
+  
+}
+
 XSCREENSAVER_MODULE_2 ("RaverHoop", raverhoop, hoop)
 
 #endif /* USE_GL */
