@@ -112,7 +112,6 @@ static const char sccsid[] = "@(#)rubik.c	5.01 2001/03/01 xlockmore";
 					"*size:  -6 \n"		\
 					"*suppressRotationAnimation: True\n" \
 
-# define refresh_rubik 0
 # define release_rubik 0
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 #else /* !STANDALONE */
@@ -165,7 +164,7 @@ ENTRYPOINT ModeSpecOpt rubik_opts =
 #ifdef USE_MODULES
 ModStruct   rubik_description =
 {"rubik", "init_rubik", "draw_rubik", (char *) NULL,
- "draw_rubik", "change_rubik", (char *) NULL, &rubik_opts,
+ "draw_rubik", "change_rubik", "free_rubik", &rubik_opts,
  10000, -30, 5, -6, 64, 1.0, "",
  "Shows an auto-solving Rubik's Cube", 0, NULL};
 
@@ -1461,7 +1460,7 @@ rotateFace(rubikstruct * rp, int face, int direction)
 		/* DrawSquare(face, position); */
 	}
 	if (faceLoc != NULL)
-		(void) free((void *) faceLoc);
+		free(faceLoc);
 	return True;
 }
 
@@ -1729,7 +1728,7 @@ shuffle(ModeInfo * mi)
 
 	for (face = 0; face < MAXFACES; face++) {
 		if (rp->cubeLoc[face] != NULL)
-			(void) free((void *) rp->cubeLoc[face]);
+			free(rp->cubeLoc[face]);
 		if ((rp->cubeLoc[face] = (RubikLoc *) malloc(sizeFace(rp, face) *
 				sizeof (RubikLoc))) == NULL) {
 			return False;
@@ -1741,7 +1740,7 @@ shuffle(ModeInfo * mi)
 	}
 	for (i = 0; i < MAXORIENT; i++) {
 		if (rp->rowLoc[i] != NULL)
-			(void) free((void *) rp->rowLoc[i]);
+			free(rp->rowLoc[i]);
 		/* The following is reused so make it the biggest size */
 		if ((rp->rowLoc[i] = (RubikLoc *) malloc(MAXMAXSIZE *
 				sizeof (RubikLoc))) == NULL) {
@@ -1751,7 +1750,7 @@ shuffle(ModeInfo * mi)
 	rp->storedmoves = MI_COUNT(mi);
 	if (rp->storedmoves < 0) {
 		if (rp->moves != NULL)
-			(void) free((void *) rp->moves);
+			free(rp->moves);
 		rp->moves = (RubikMove *) NULL;
 		rp->storedmoves = NRAND(-rp->storedmoves) + 1;
 	}
@@ -1814,8 +1813,14 @@ ENTRYPOINT void
 reshape_rubik(ModeInfo * mi, int width, int height)
 {
 	rubikstruct *rp = &rubik[MI_SCREEN(mi)];
+    int y = 0;
 
-	glViewport(0, 0, rp->WindW = (GLint) width, rp->WindH = (GLint) height);
+    if (width > height * 5) {   /* tiny window: show middle */
+      height = width;
+      y = -height/2;
+    }
+
+	glViewport(0, y, rp->WindW = (GLint) width, rp->WindH = (GLint) height);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0);
@@ -1869,24 +1874,29 @@ pinit(ModeInfo * mi)
 	return (shuffle(mi));
 }
 
-static void
+ENTRYPOINT void
 free_rubik(ModeInfo *mi)
 {
 	rubikstruct *rp = &rubik[MI_SCREEN(mi)];
 	int         i;
 
+    if (!rp->glx_context) return;
+	glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *rp->glx_context);
+
+    if (rp->trackball) gltrackball_free (rp->trackball);
+
 	for (i = 0; i < MAXFACES; i++)
 		if (rp->cubeLoc[i] != NULL) {
-			(void) free((void *) rp->cubeLoc[i]);
+			free(rp->cubeLoc[i]);
 			rp->cubeLoc[i] = (RubikLoc *) NULL;
 		}
 	for (i = 0; i < MAXORIENT; i++)
 		if (rp->rowLoc[i] != NULL) {
-			(void) free((void *) rp->rowLoc[i]);
+			free(rp->rowLoc[i]);
 			rp->rowLoc[i] = (RubikLoc *) NULL;
 		}
 	if (rp->moves != NULL) {
-		(void) free((void *) rp->moves);
+		free(rp->moves);
 		rp->moves = (RubikMove *) NULL;
 	}
 }
@@ -1896,7 +1906,7 @@ init_rubik(ModeInfo * mi)
 {
 	rubikstruct *rp;
 
-	MI_INIT (mi, rubik, free_rubik);
+	MI_INIT (mi, rubik);
 	rp = &rubik[MI_SCREEN(mi)];
 	rp->step = NRAND(90);
 	rp->PX = ((float) LRAND() / (float) MAXRAND) * 2.0 - 1.0;
@@ -1940,7 +1950,7 @@ draw_rubik(ModeInfo * mi)
 		return;
 
     mi->polygon_count = 0;
-	glXMakeCurrent(display, window, *(rp->glx_context));
+	glXMakeCurrent(display, window, *rp->glx_context);
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
