@@ -1,4 +1,4 @@
-/* Sky Tentacles, Copyright (c) 2008-2014 Jamie Zawinski <jwz@jwz.org>
+/* Sky Tentacles, Copyright (c) 2008-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -14,7 +14,6 @@
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
 
-# define free_tentacles 0
 # define release_tentacles 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -26,16 +25,8 @@
 #include "gltrackball.h"
 #include <ctype.h>
 
-#include "xpm-ximage.h"
-#include "../images/scales.xpm"
-
-static char *grey_texture[] = {
-  "16 1 3 1",
-  "X c #808080",
-  "x c #C0C0C0",
-  ". c #FFFFFF",
-  "XXXxxxxx........"
-};
+#include "ximage-loader.h"
+#include "images/gen/scales_png.h"
 
 #ifdef USE_GL /* whole file */
 
@@ -98,7 +89,6 @@ typedef struct {
   GLuint texid;
 
   Bool left_p;
-  
 
 } tentacles_configuration;
 
@@ -919,9 +909,27 @@ init_tentacles (ModeInfo *mi)
     glBindTexture ((cel_p ? GL_TEXTURE_1D : GL_TEXTURE_2D), tc->texid);
 # endif
 
-    tc->texture = xpm_to_ximage (MI_DISPLAY(mi), MI_VISUAL(mi), 
-                                 MI_COLORMAP(mi), 
-                                 (cel_p ? grey_texture : scales));
+    if (cel_p)
+      {
+        /* "16 1 3 1",
+           "X c #808080",
+           "x c #C0C0C0",
+           ". c #FFFFFF",
+           "XXXxxxxx........"
+         */
+        int w = 16;
+        tc->texture = XCreateImage (MI_DISPLAY(mi), MI_VISUAL(mi),
+                                    32, ZPixmap, 0, 0, w, 1, 32, 0);
+        tc->texture->data = (char *) calloc(1, tc->texture->bytes_per_line);
+        /* ABGR */
+        for (i = 0; i < 3; i++) XPutPixel (tc->texture, i, 0, 0xFF808080);
+        for (;      i < 8; i++) XPutPixel (tc->texture, i, 0, 0xFFC0C0C0);
+        for (;      i < w; i++) XPutPixel (tc->texture, i, 0, 0xFFFFFFFF);
+      }
+    else
+      tc->texture = image_data_to_ximage (MI_DISPLAY(mi), MI_VISUAL(mi), 
+                                          scales_png, sizeof(scales_png));
+
     if (!tc->texture) texture_p = cel_p = False;
   }
 
@@ -930,10 +938,7 @@ init_tentacles (ModeInfo *mi)
     clear_gl_error();
     glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA,
                   tc->texture->width, tc->texture->height, 0,
-                  GL_RGBA,
-                  /* GL_UNSIGNED_BYTE, */
-                  GL_UNSIGNED_INT_8_8_8_8_REV,
-                  tc->texture->data);
+                  GL_RGBA, GL_UNSIGNED_BYTE, tc->texture->data);
     check_gl_error("texture");
 
     glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
@@ -949,10 +954,7 @@ init_tentacles (ModeInfo *mi)
     clear_gl_error();
     glTexImage1D (GL_TEXTURE_1D, 0, GL_RGBA,
                   tc->texture->width, 0,
-                  GL_RGBA,
-                  /* GL_UNSIGNED_BYTE, */
-                  GL_UNSIGNED_INT_8_8_8_8_REV,
-                  tc->texture->data);
+                  GL_RGBA, GL_UNSIGNED_BYTE, tc->texture->data);
     check_gl_error("texture");
 
     glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_REPEAT);
@@ -990,7 +992,7 @@ draw_tentacles (ModeInfo *mi)
   if (!tc->glx_context)
     return;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(tc->glx_context));
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *tc->glx_context);
 
   glShadeModel(GL_SMOOTH);
 
@@ -1094,6 +1096,30 @@ draw_tentacles (ModeInfo *mi)
   glXSwapBuffers(dpy, window);
 }
 
+
+ENTRYPOINT void
+free_tentacles (ModeInfo *mi)
+{
+  tentacles_configuration *tc = &tcs[MI_SCREEN(mi)];
+  int i, j;
+
+  if (!tc->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *tc->glx_context);
+
+  for (i = 0; i < tc->ntentacles; i++) {
+    for (j = 0; j < tc->tentacles[i]->nsegments; j++)
+      free_rotator (tc->tentacles[i]->segments[j].rot);
+    free (tc->tentacles[i]->segments);
+    free (tc->tentacles[i]);
+  }
+  if (tc->tentacles) free (tc->tentacles);
+  if (tc->torus_points) free (tc->torus_points);
+  if (tc->torus_normals) free (tc->torus_normals);
+  if (tc->texture) XDestroyImage (tc->texture);
+  if (tc->trackball) gltrackball_free (tc->trackball);
+  if (tc->texid) glDeleteTextures (1, &tc->texid);
+}
+
 XSCREENSAVER_MODULE_2 ("SkyTentacles", skytentacles, tentacles)
 
 #endif /* USE_GL */
