@@ -1,4 +1,4 @@
-/* starwars, Copyright (c) 1998-2015 Jamie Zawinski <jwz@jwz.org> and
+/* starwars, Copyright (c) 1998-2018 Jamie Zawinski <jwz@jwz.org> and
  * Claudio Matsuoka <claudio@helllabs.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -568,6 +568,12 @@ init_stars (ModeInfo *mi, int width, int height)
   int max_size = 3;
   GLfloat inc = 0.5;
   int steps = max_size / inc;
+  GLfloat scale = 1;
+
+  if (MI_WIDTH(mi) > 2560) {  /* Retina displays */
+    scale *= 2;
+    nstars = (size/scale) * (size/scale) / 320;
+  }
 
   glDeleteLists (sc->star_list, 1);
   sc->star_list = glGenLists (1);
@@ -577,7 +583,7 @@ init_stars (ModeInfo *mi, int width, int height)
 
   for (j = 1; j <= steps; j++)
     {
-      glPointSize(inc * j);
+      glPointSize(inc * j * scale);
       glBegin (GL_POINTS);
       for (i = 0; i < nstars / steps; i++)
         {
@@ -890,7 +896,7 @@ draw_sws (ModeInfo *mi)
     return;
 
   glDrawBuffer (GL_BACK);
-  glXMakeCurrent (dpy, window, *(sc->glx_context));
+  glXMakeCurrent (dpy, window, *sc->glx_context);
 
   glClear (GL_COLOR_BUFFER_BIT);
 
@@ -1051,10 +1057,18 @@ ENTRYPOINT void
 free_sws (ModeInfo *mi)
 {
   sws_configuration *sc = &scs[MI_SCREEN(mi)];
-  if (sc->tc)
-    textclient_close (sc->tc);
-
-  /* #### there's more to free here */
+  int i;
+  if (!sc->glx_context) return;
+  glXMakeCurrent (MI_DISPLAY(mi), MI_WINDOW(mi), *sc->glx_context);
+  if (sc->tc) textclient_close (sc->tc);
+  if (sc->texfont) free_texture_font (sc->texfont);
+  if (sc->buf) free (sc->buf);
+  if (sc->line_widths) free (sc->line_widths);
+  for (i = 0; i < sc->total_lines; i++)
+    if (sc->lines[i]) free (sc->lines[i]);
+  if (sc->lines) free (sc->lines);
+  if (glIsList(sc->star_list)) glDeleteLists(sc->star_list, 1);
+  if (glIsList(sc->text_list)) glDeleteLists(sc->text_list, 1);
 }
 
 
