@@ -46,7 +46,6 @@
                                     "*showFPS:      False   \n" \
 				   "*suppressRotationAnimation: True\n" \
 
-# define refresh_surface 0
 # define release_surface 0
 # include "xlockmore.h"     /* from the xscreensaver distribution */
 #else  /* !STANDALONE */
@@ -421,10 +420,17 @@ ENTRYPOINT void reshape_surface(ModeInfo *mi, int width, int height)
 {
   surfacestruct *sp = &surface[MI_SCREEN(mi)];
   GLfloat h = (GLfloat) height / (GLfloat) width;
+  int y = 0;
 
-  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sp->glx_context));
+  if (width > height * 5) {   /* tiny window: show middle */
+    height = width * 9/16;
+    y = -height/2;
+    h = height / (GLfloat) width;
+  }
 
-  glViewport(0, 0, (GLint) width, (GLint) height);
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *sp->glx_context);
+
+  glViewport(0, y, (GLint) width, (GLint) height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective (30.0, 1/h, 1.0, 100.0);
@@ -463,7 +469,7 @@ ENTRYPOINT void init_surface(ModeInfo *mi)
   int    screen = MI_SCREEN(mi);
   surfacestruct *sp;
 
-  MI_INIT (mi, surface, NULL);
+  MI_INIT (mi, surface);
   sp = &surface[screen];
 
   sp->window = MI_WINDOW(mi);
@@ -627,7 +633,7 @@ ENTRYPOINT void draw_surface(ModeInfo * mi)
 
   glDrawBuffer(GL_BACK);
 
-  glXMakeCurrent(display, window, *(sp->glx_context));
+  glXMakeCurrent(display, window, *sp->glx_context);
   draw(mi);
   if (mi->fps_p)
     do_fps(mi);
@@ -636,6 +642,16 @@ ENTRYPOINT void draw_surface(ModeInfo * mi)
 }
 
 
+ENTRYPOINT void free_surface(ModeInfo * mi)
+{
+  surfacestruct *sp = &surface[MI_SCREEN(mi)];
+  if (!sp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *sp->glx_context);
+  gltrackball_free (sp->trackball);
+  free_rotator (sp->rot);
+}
+
+
 XSCREENSAVER_MODULE_2("Surfaces", surfaces, surface)
 
 #endif
