@@ -22,7 +22,6 @@
 	"*titleFont2: -*-helvetica-medium-r-normal-*-*-120-*-*-*-*-*-*\n" \
 	"*titleFont3: -*-helvetica-medium-r-normal-*-*-80-*-*-*-*-*-*\n"  \
 
-# define refresh_tangram 0
 # define release_tangram 0
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
@@ -779,7 +778,14 @@ static void draw_shapes(ModeInfo * mi)
 
 ENTRYPOINT void reshape_tangram(ModeInfo * mi, int w, int h)
 {
-    glViewport(0, 0, w, h);
+  int y = 0;
+
+  if (w > h * 5) {   /* tiny window: show middle */
+    h = w;
+    y = -h/2;
+  }
+
+  glViewport(0, y, w, h);
 }
 
 static void set_camera(tangram_configuration *tp)
@@ -877,7 +883,7 @@ ENTRYPOINT void init_tangram(ModeInfo * mi)
 {
     tangram_configuration *tp;
 
-    MI_INIT (mi, tps, NULL);
+    MI_INIT (mi, tps);
 
     tp = &tps[MI_SCREEN(mi)];
 
@@ -960,7 +966,7 @@ ENTRYPOINT void draw_tangram(ModeInfo * mi)
 
     if (! tp->glx_context)
       return;
-    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(tp->glx_context));
+    glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *tp->glx_context);
 
     ls[small_triangle1] = &tp->tsm1;
     ls[small_triangle2] = &tp->tsm2;
@@ -1053,7 +1059,16 @@ tangram_handle_event (ModeInfo *mi, XEvent *event)
   return False;
 }
 
-
+ENTRYPOINT void
+free_tangram (ModeInfo *mi)
+{
+  tangram_configuration *tp = &tps[MI_SCREEN(mi)];
+  if (!tp->glx_context) return;
+  glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *tp->glx_context);
+  if (tp->font1_data) free_texture_font (tp->font1_data);
+  if (tp->font2_data) free_texture_font (tp->font2_data);
+  if (tp->font3_data) free_texture_font (tp->font3_data);
+}
 
 XSCREENSAVER_MODULE ("Tangram", tangram)
 
