@@ -85,7 +85,7 @@ static void update_buffer(halftone_screen *halftone, XWindowAttributes * attrs)
 
     halftone->buffer_width = attrs->width;
     halftone->buffer_height = attrs->height;
-#ifdef HAVE_COCOA	/* Don't second-guess Quartz's double-buffering */
+#ifdef HAVE_JWXYZ	/* Don't second-guess Quartz's double-buffering */
     halftone->buffer = halftone->window;
 #else
     halftone->buffer = XCreatePixmap(halftone->dpy, halftone->window, halftone->buffer_width, halftone->buffer_height, attrs->depth);
@@ -357,6 +357,17 @@ static void
 halftone_free (Display *dpy, Window window, void *closure)
 {
   halftone_screen *halftone = (halftone_screen *) closure;
+  free (halftone->gravity_point_x);
+  free (halftone->gravity_point_y);
+  free (halftone->gravity_point_mass);
+  free (halftone->gravity_point_x_inc);
+  free (halftone->gravity_point_y_inc);
+  free (halftone->dots);
+  free (halftone->colors);
+  if (halftone->buffer && halftone->buffer != halftone->window)
+    XFreePixmap (dpy, halftone->buffer);
+  if (halftone->gc) XFreeGC (dpy, halftone->gc);
+  if (halftone->buffer_gc) XFreeGC (dpy, halftone->buffer_gc);
   free (halftone);
 }
 
@@ -373,7 +384,7 @@ static const char *halftone_defaults [] = {
   "*sizeFactor:		1.5",
   "*colors:		200",
   "*cycleSpeed:		10",
-#ifdef USE_IPHONE
+#ifdef HAVE_MOBILE
   "*ignoreRotation:     True",
 #endif
   0
