@@ -364,10 +364,8 @@ static void inter_free(Display* dpy, struct inter_context* c)
 
   destroy_image(dpy, c);
 
-  if(c->colors <= 2)
-    free(c->pal);
-  else if(c->pal)
-    free_colors(c->screen, c->cmap, c->pal, c->colors);
+  free_colors(c->screen, c->cmap, c->pal, c->colors);
+  if (c->pal) free(c->pal);
 
 #ifndef USE_XIMAGE
   for(i = 0; i != c->colors; ++i)
@@ -696,6 +694,7 @@ static void inter_init(Display* dpy, Window win, struct inter_context* c)
   int mono;
   int gray;
   int radius;
+  double scale = 1;
   XGCValues val;
   Bool dbuf = get_boolean_resource (dpy, "doubleBuffer", "Boolean");
 
@@ -749,6 +748,9 @@ static void inter_init(Display* dpy, Window win, struct inter_context* c)
   if(radius < 1)
     radius = 1;
 
+  if (xgwa.width > 2560) scale = 3.5;  /* Retina displays */
+  radius *= scale;
+
   create_image(dpy, c, &xgwa);
 
   if(!mono) {
@@ -843,7 +845,7 @@ static void inter_init(Display* dpy, Window win, struct inter_context* c)
       ((float)radius);
     c->wave_height[i] = 
       (unsigned)
-      ((max + max*cos(fi/50.0)) / 2.0);
+      ((max + max*cos(fi/(50.0 * scale))) / 2.0);
   }
 
   c->source = calloc(c->count, sizeof(struct inter_source));
@@ -991,6 +993,7 @@ interference_free (Display *dpy, Window window, void *closure)
 {
   struct inter_context *c = (struct inter_context *) closure;
   inter_free(dpy, c);
+  free(c);
 }
 
 XSCREENSAVER_MODULE ("Interference", interference)
