@@ -254,6 +254,10 @@ static void Initialize( struct state *st )
         if (st->radius >= 128) /* should use UCHAR_MAX? */
           st->radius = 127; /* st->dradius should fit in u_char */
 
+        if (st->iWinWidth < 100 || st->iWinHeight < 100) /* tiny window */
+          if (st->radius < 20)
+            st->radius = 20;
+
 	st->dradius = st->radius * 2;
 	st->sradius = st->radius * st->radius;
 
@@ -382,19 +386,20 @@ metaballs_event (Display *dpy, Window window, void *closure, XEvent *event)
 static void
 metaballs_free (Display *dpy, Window window, void *closure)
 {
-#if 0
   struct state *st = (struct state *) closure;
-	free( st->pImage->data );
-	XDestroyImage( st->pImage );
-	free( st->aiColorVals );
-	free( st->blobs );
-	for (i = 0; i < st->iWinHeight; ++i)
-	  free( st->blub[i] );
-	free( st->blub );
-	for (i = 0; i < st->dradius; ++i)
-	  free( st->blob[i] );
-	free( st->blob );
-#endif
+  int i;
+  if (st->pImage) XDestroyImage (st->pImage);
+  free (st->aiColorVals);
+  free (st->blobs);
+  for (i = 0; i < st->iWinHeight; ++i)
+    free (st->blub[i]);
+  free (st->blub);
+  for (i = 0; i < st->dradius; ++i)
+    free (st->blob[i]);
+  if (st->sColor) free (st->sColor);
+  free (st->blob);
+  XFreeGC (dpy, st->gc);
+  free (st);
 }
 
 
