@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1999-2017 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1999-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -55,33 +55,18 @@
 
 #include "screenhack.h"
 #include "textclient.h"
-#include "xpm-pixmap.h"
+#include "ximage-loader.h"
 #include <stdio.h>
 #include <sys/wait.h>
 
-#ifdef HAVE_JWXYZ
-# define HAVE_XPM
-#else
-# define DO_XBM     /* only do mono bitmaps under real X11 */
-#endif
-
 #ifndef HAVE_JWXYZ
 # include <X11/Intrinsic.h>
 #endif
 
-#if defined(HAVE_GDK_PIXBUF) || defined(HAVE_XPM)
-# include "images/matrix1.xpm"
-# include "images/matrix2.xpm"
-# include "images/matrix1b.xpm"
-# include "images/matrix2b.xpm"
-#endif
-
-#ifdef DO_XBM
-# include "images/matrix1.xbm"
-# include "images/matrix2.xbm"
-# include "images/matrix1b.xbm"
-# include "images/matrix2b.xbm"
-#endif /* DO_XBM */
+#include "images/gen/matrix1_png.h"
+#include "images/gen/matrix2_png.h"
+#include "images/gen/matrix1b_png.h"
+#include "images/gen/matrix2b_png.h"
 
 #define CHAR_COLS 16
 #define CHAR_ROWS 13
@@ -234,39 +219,25 @@ typedef struct {
 static void
 load_images_1 (Display *dpy, m_state *state, int which)
 {
-#if defined(HAVE_GDK_PIXBUF) || defined(HAVE_XPM)
-  if (!get_boolean_resource (dpy, "mono", "Boolean") &&
-      state->xgwa.depth > 1)
+  const unsigned char *png = 0;
+  unsigned long size = 0;
+  if (which == 1)
     {
-      char **bits =
-        (which == 1 ? (state->small_p ? matrix1b_xpm : matrix1_xpm) :
-         (state->small_p ? matrix2b_xpm : matrix2_xpm));
-
-      state->images[which] =
-        xpm_data_to_pixmap (state->dpy, state->window, bits,
-                            &state->image_width, &state->image_height, 0);
+      if (state->small_p)
+        png = matrix1b_png, size = sizeof(matrix1b_png);
+      else
+        png = matrix1_png, size = sizeof(matrix1_png);
     }
   else
-#endif /* !HAVE_XPM && !HAVE_GDK_PIXBUF */
     {
-#ifdef DO_XBM
-      unsigned long fg, bg;
-      state->image_width  = (state->small_p ? matrix1b_width :matrix1_width);
-      state->image_height = (state->small_p ? matrix1b_height:matrix1_height);
-      fg = get_pixel_resource(state->dpy, state->xgwa.colormap,
-                              "foreground", "Foreground");
-      bg = get_pixel_resource(state->dpy, state->xgwa.colormap,
-                              "background", "Background");
-      state->images[which] =
-        XCreatePixmapFromBitmapData (state->dpy, state->window, (char *)
-                (which == 1 ? (state->small_p ? matrix1b_bits :matrix1_bits) :
-                              (state->small_p ? matrix2b_bits :matrix2_bits)),
-                                     state->image_width, state->image_height,
-                                     bg, fg, state->xgwa.depth);
-#else  /* !DO_XBM */
-      abort();
-#endif /* !DO_XBM */
+      if (state->small_p)
+        png = matrix2b_png, size = sizeof(matrix2b_png);
+      else
+        png = matrix2_png, size = sizeof(matrix2_png);
     }
+  state->images[which] =
+    image_data_to_pixmap (state->dpy, state->window, png, size,
+                          &state->image_width, &state->image_height, 0);
 }
 
 
@@ -466,6 +437,7 @@ init_trace (m_state *state)
   if (!s)
     goto FAIL;
 
+  if (state->tracing) free (state->tracing);
   state->tracing = (signed char *) malloc (strlen (s) + 1);
   s3 = state->tracing;
 
@@ -479,6 +451,7 @@ init_trace (m_state *state)
 
   state->glyph_map = decimal_encoding;
   state->nglyphs = countof(decimal_encoding);
+  free (s);
 
   return;
 
@@ -609,7 +582,7 @@ xmatrix_init (Display *dpy, Window window)
 
   state->small_p = (state->xgwa.width < 300);
   {
-    const char *s = get_string_resource (dpy, "matrixFont", "String");
+    char *s = get_string_resource (dpy, "matrixFont", "String");
     if (!s || !*s || !strcasecmp(s, "large"))
       state->small_p = False;
     else if (!strcasecmp(s, "small"))
@@ -617,6 +590,7 @@ xmatrix_init (Display *dpy, Window window)
     else
       fprintf (stderr, "%s: matrixFont should be 'small' or 'large' not '%s'\n",
                progname, s);
+    if (s) free (s);
   }
 
   load_images (dpy, state);
@@ -762,6 +736,7 @@ xmatrix_init (Display *dpy, Window window)
       fprintf (stderr, "not `%s'\n", mode);
       set_mode (state, MATRIX);
     }
+  if (mode) free (mode);
 
   if (state->mode == MATRIX && get_boolean_resource (dpy, "trace", "Boolean"))
     set_mode (state, ((random() % 3) ? TRACE_TEXT_A : TRACE_TEXT_B));
@@ -1824,19 +1799,27 @@ static void
 xmatrix_free (Display *dpy, Window window, void *closure)
 {
   m_state *state = (m_state *) closure;
+  int i;
   if (state->tc)
     textclient_close (state->tc);
   if (state->cursor_timer)
     XtRemoveTimeOut (state->cursor_timer);
-
-  /* #### there's more to free here */
-
+  XFreeGC (dpy, state->draw_gc);
+  XFreeGC (dpy, state->erase_gc);
+  XFreeGC (dpy, state->scratch_gc);
+  free (state->cells);
+  free (state->background);
+  free (state->feeders);
+  if (state->tracing) free (state->tracing);
+  for (i = 0; i < CHAR_MAPS; i++)
+    if (state->images[i]) XFreePixmap (dpy, state->images[i]);
   free (state);
 }
 
 static const char *xmatrix_defaults [] = {
   ".background:		   black",
   ".foreground:		   #00AA00",
+  ".lowrez:		   true",  /* Small font is unreadable at 5120x2880 */
   "*fpsSolid:		   true",
   "*matrixFont:		   large",
   "*delay:		   10000",
