@@ -83,7 +83,7 @@ implied warranty.
 #	include <unistd.h>
 #endif
 
-#ifdef HAVE_COCOA
+#if defined HAVE_JWXYZ
 #	include "jwxyz.h"
 #else
 #	include <X11/Xlib.h>
